#include <windows.h>
#include <shlobj.h>
#include <stdio.h>
#include <tchar.h>
#include <atlstr.h>

#pragma comment(lib, "shell32")

void CleanupPath(LPCTSTR path, LPCTSTR pathDelTo)
{
	for (CString _path = path; _path.CompareNoCase(pathDelTo); )
	{
		RemoveDirectory(_path);

		PathRemoveFileSpec(_path.GetBuffer());
		_path.ReleaseBuffer();
	}
}

BOOL Touch(LPCTSTR filePath)
{
	HANDLE hFile = CreateFile(filePath, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
	CloseHandle(hFile);
	return (INVALID_HANDLE_VALUE != hFile);
}

void TestPath(CString path, CString pathDelTo, BOOL& folderExists, BOOL& folderCreatable, BOOL& folderWritable)
{
	DWORD attr = GetFileAttributes(pathDelTo);
	if (DWORD(-1) == attr || !(FILE_ATTRIBUTE_DIRECTORY & attr))
	{
		folderExists = FALSE;
		return;
	}

	if (pathDelTo.CompareNoCase(path.Left(pathDelTo.GetLength())))
	{
		folderExists = FALSE;
		return;
	}

	folderExists = TRUE;

	attr = GetFileAttributes(path);
	if (DWORD(-1) != attr && FILE_ATTRIBUTE_DIRECTORY & attr)
	{
		DeleteFile(path + TEXT("\\MyAppName.exe"));
		CleanupPath(path, pathDelTo);
		Sleep(40);
	}

	int res = SHCreateDirectory(NULL, CStringW(path));
	switch (res)
	{
	case ERROR_PATH_NOT_FOUND:
		break;

	case ERROR_FILE_EXISTS:
	case ERROR_ALREADY_EXISTS:
		_tprintf(TEXT("Already exists.\n"));
		break;

	case ERROR_FILENAME_EXCED_RANGE:
		_tprintf(TEXT("Error: path length exceeds MAX_PATH limit, (%s) %S"), _tcslen(path), path);
	case ERROR_BAD_PATHNAME:
		_tprintf(TEXT("Error: bad path %s"), path);
		break;

	case ERROR_SUCCESS:
		folderCreatable = TRUE;
		if (Touch(path + TEXT("\\MyAppName.exe")))
			folderWritable = TRUE;
		break;
	}
}

int _tmain(int argc, TCHAR** argv)
{
	LPCTSTR tests[] = 
	{
		TEXT("%APPDATA%"),
		TEXT("%HOME%"),
		TEXT("%USERPROFILE%"),
		TEXT("%PUBLIC%"),
		TEXT("%ProgramFiles%"),
		TEXT("%ProgramFiles(x86)%"),
		TEXT("%CommonProgramFiles%"),
		TEXT("%CommonProgramFiles(x86)%"),
		TEXT("%SystemRoot%"),
		TEXT("%ProgramData%"),
		TEXT("%TEMP%"),
		NULL
	};

	BOOL justCleanUp = FALSE;
	if (0 == CString(argv[1]).CompareNoCase(TEXT("clean")))
		justCleanUp = TRUE;

	for (int idx = 0; tests[idx]; idx++)
	{
		_tprintf(TEXT("%-30s"), tests[idx]);
		TCHAR pathEnv[MAX_PATH] = {0};
		if (0 == ExpandEnvironmentStrings(tests[idx], pathEnv, MAX_PATH)
			|| 0 == _tcslen(pathEnv))
		{
			_tprintf(TEXT("\n"));
			continue;
		}
		_tprintf(TEXT(" %s\n"), pathEnv);
		CString pathDelTo = pathEnv;
		CString path = pathDelTo + TEXT("\\MyCompany\\MyAppName");

		if (justCleanUp)
		{
			DeleteFile(path + TEXT("\\MyAppName.exe"));
			CleanupPath(path, pathDelTo);
		}
		else
		{
			BOOL folderExists = FALSE, folderCreatable = FALSE, folderWritable = FALSE;
			TestPath(path, pathDelTo, folderExists, folderCreatable, folderWritable);

			_tprintf(TEXT("%30s * exists = %d, creatable = %d, writable = %d\n"), "", 
				folderExists, folderCreatable, folderWritable);
		}
	}

	return 0;
}